package com.company.ser.core;

import com.company.ser.SerTestContainer;
import com.company.ser.entity.City;
import com.company.ser.entity.Company;
import com.company.ser.entity.Order;
import com.haulmont.cuba.core.EntityManager;
import com.haulmont.cuba.core.Persistence;
import com.haulmont.cuba.core.Transaction;
import com.haulmont.cuba.core.TypedQuery;
import com.haulmont.cuba.core.app.serialization.EntitySerializationAPI;
import com.haulmont.cuba.core.global.AppBeans;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.Metadata;
import com.haulmont.cuba.security.entity.User;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.RegisterExtension;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

public class SampleIntegrationTest {

    @RegisterExtension
    public static SerTestContainer cont = SerTestContainer.Common.INSTANCE;

    private static Metadata metadata;
    private static Persistence persistence;
    private static DataManager dataManager;

    private static EntitySerializationAPI entitySerializationAPI;

    @BeforeAll
    public static void beforeAll() throws Exception {
        metadata = cont.metadata();
        persistence = cont.persistence();
        dataManager = AppBeans.get(DataManager.class);
        entitySerializationAPI = AppBeans.get(EntitySerializationAPI.class);
    }

    @AfterAll
    public static void afterAll() throws Exception {
    }

    @Test
    public void testLoadUser() {
        try (Transaction tx = persistence.createTransaction()) {
            EntityManager em = persistence.getEntityManager();
            TypedQuery<User> query = em.createQuery(
                    "select u from sec$User u where u.login = :userLogin", User.class);
            query.setParameter("userLogin", "admin");
            List<User> users = query.getResultList();
            tx.commit();
            assertEquals(1, users.size());
        }
    }

    @Test
    public void testDeserialization() {
        String json = "  {\n" +
                "    \"_entityName\": \"ser_Order\",\n" +
                "    \"id\": \"c87a6bee-8f00-243e-f8cf-a0c002c339e1\",\n" +
                "    \"createdBy\": \"admin\",\n" +
                "    \"price\": 1001.00,\n" +
                "    \"name\": \"A-001\",\n" +
                "    \"createTs\": \"2023-11-14 13:31:08.009\",\n" +
                "    \"company\":   {\n" +
                "    \"_entityName\": \"ser_Company\",\n" +
                "    \"id\": \"c1a04341-0700-baa8-fed2-172790863f96\",\n" +
                "    \"address\": \"NY city, NY\",\n" +
                "    \"createdBy\": \"admin\",\n" +
                "    \"city\": {\n" +
                "      \"_entityName\": \"ser_City\",\n" +
                "      \"id\": \"4417bbb5-8135-4188-5092-05a5076ea5bc\",\n" +
                "      \"code\": \"NY\",\n" +
                "      \"createdBy\": \"admin\",\n" +
                "      \"name\": \"New York\",\n" +
                "      \"createTs\": \"2023-11-14 13:30:41.866\",\n" +
                "      \"version\": 1,\n" +
                "      \"updateTs\": \"2023-11-14 13:30:41.866\"\n" +
                "    },\n" +
                "    \"name\": \"Amazon\",\n" +
                "    \"createTs\": \"2023-11-14 13:30:55.236\",\n" +
                "    \"version\": 1,\n" +
                "    \"updateTs\": \"2023-11-14 13:30:55.236\"\n" +
                "  },\n" +
                "    \"version\": 1,\n" +
                "    \"updateTs\": \"2023-11-14 13:31:08.009\"\n" +
                "  }";

        Order order = entitySerializationAPI.entityFromJson(json, metadata.getClass(Order.class));

        Company company = order.getCompany();
        assertNotNull(company);
        assertEquals("Amazon", company.getName());
        assertEquals("NY city, NY", company.getAddress());

        City city = company.getCity();
        assertNotNull(city);
        assertEquals("New York", city.getName());
        assertEquals("NY", city.getCode());
    }
}